@@ -20,8 +20,8 @@
 #   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
-VERSION = 33
-VERSION_DATE = '2012-09-11'
+VERSION = 36
+VERSION_DATE = '2013-10-11'
 
 import logging
 logger = logging.getLogger('cachedownloader')
@@ -31,13 +31,14 @@ try:
 except (ImportError, AttributeError):
     import simplejson as json	 
 from geocaching import GeocacheCoordinate
+from downloader import read_from_network
 import geo
 import os
 import threading
 import re
 import gobject
 from utils import HTMLManipulations
-from lxml.html import fromstring, tostring, submit_form
+from lxml.html import fromstring, tostring
 
 #ugly workaround...
 user_token = [None]
@@ -113,17 +114,14 @@ class CacheDownloader(gobject.GObject):
         return points
         
     # Upload one or more fieldnotes
-    def upload_fieldnotes(self, geocaches, upload_as_logs = False):
-    
+    def upload_fieldnotes_and_logs(self, geocaches):
         try:
-            if not upload_as_logs:
-                self._upload_fieldnotes(geocaches)
-            else:
-                self._upload_logs(geocaches)
+            success = self._upload_fieldnotes_and_logs(geocaches)
         except Exception, e:
+            logger.exception(e)
             self.emit('download-error', e)
             return []
-        return geocaches
+        return success
                 
         
 class GeocachingComCacheDownloader(CacheDownloader):
@@ -132,7 +130,6 @@ class GeocachingComCacheDownloader(CacheDownloader):
 
     MAX_DOWNLOAD_NUM = 800
 
-
     CTIDS = {
         2:GeocacheCoordinate.TYPE_REGULAR,
         3:GeocacheCoordinate.TYPE_MULTI,
@@ -143,16 +140,30 @@ class GeocachingComCacheDownloader(CacheDownloader):
         137:GeocacheCoordinate.TYPE_EARTH
     }
     
+    TRANS_FIELDNOTE_TYPE = {
+        GeocacheCoordinate.LOG_AS_FOUND: "Found it",
+        GeocacheCoordinate.LOG_AS_NOTFOUND: "Didn't find it",
+        GeocacheCoordinate.LOG_AS_NOTE: "Write note"
+    }
+        
+       
+    TRANS_LOG_TYPE = {
+        GeocacheCoordinate.LOG_AS_FOUND: 2,
+        GeocacheCoordinate.LOG_AS_NOTFOUND: 3,
+        GeocacheCoordinate.LOG_AS_NOTE: 4
+    }
+    
     # URL for log pages; fetches 10 logs by default
     LOGBOOK_URL = 'http://www.geocaching.com/seek/geocache.logbook?tkn=%s&idx=%d&num=10&decrypt=true'
     OVERVIEW_URL = 'http://www.geocaching.com/seek/nearest.aspx?lat=%f&lng=%f&dist=%f'
-    PRINT_PREVIEW_URL = 'http://www.geocaching.com/seek/cdpf.aspx?guid=%s'
+    PRINT_PREVIEW_URL_FULLNAME = 'http://www.geocaching.com/geocache/%s'
     SEEK_URL = "http://www.geocaching.com/seek/%s"
     DETAILS_URL = 'http://www.geocaching.com/seek/cache_details.aspx?wp=%s'
     LOGIN_URL = 'https://www.geocaching.com/login/default.aspx'
     NEAREST_URL = 'http://www.geocaching.com/seek/nearest.aspx'
     USER_TOKEN_URL = 'http://www.geocaching.com/map/default.aspx?lat=6&lng=9'
     UPLOAD_FIELDNOTES_URL = 'http://www.geocaching.com/my/uploadfieldnotes.aspx'
+    UPLOAD_LOG_URL = 'http://www.geocaching.com/seek/log.aspx?wp=%s'
     
     def __init__(self, downloader, path = None, download_images = True):
         CacheDownloader.__init__(self, downloader, path, download_images)
@@ -179,40 +190,47 @@ class GeocachingComCacheDownloader(CacheDownloader):
         
         self.emit("progress", "Fetching list", 0, 1)
         response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
-        
+        if response == None:
+            logger.warning("Got None from downloader")
+
         cont = True
         wpts = []
         page_last = 0 # Stores the "old" value of the page counter; If it doesn't increment, abort!
         while cont:
             # Count the number of results and pages
-            text = response.read()
-            response.close()
-            doc = self._parse(text)
-            bs = doc.cssselect('#ctl00_ContentBody_ResultsPanel .PageBuilderWidget b')
-            if len(bs) == 0:
-                raise Exception("There are no geocaches in this area.")
-            count = int(bs[0].text_content())
-            page_current = int(bs[1].text_content())
-            if page_current == page_last:
-                raise Exception("Current page has the same number as the last page; aborting!")
-                break
-            page_last = page_current
-            page_max = int(bs[2].text_content())
-            logger.info("We are at page %d of %d, total %d geocaches" % (page_current, page_max, count))
-            if count > self.MAX_DOWNLOAD_NUM:
-                raise Exception("%d geocaches found, please select a smaller part of the map!" % count)
+            if response == None:
+                logger.warning("_get_overview: Error: response is None:"+str(e))
+            else:
+                text = read_from_network(response)
+                if text != None:
+                    doc = self._parse(text)
+                    bs = doc.cssselect('#ctl00_ContentBody_ResultsPanel .PageBuilderWidget b')
+                    if len(bs) == 0:
+                        raise Exception("There are no geocaches in this area.")
+                    count = int(bs[0].text_content())
+                    page_current = int(bs[1].text_content())
+                    if page_current == page_last:
+                        raise Exception("Current page has the same number as the last page; aborting!")
+                        break
+                    page_last = page_current
+                    page_max = int(bs[2].text_content())
+                    logger.info("We are at page %d of %d, total %d geocaches" % (page_current, page_max, count))
+                    if count > self.MAX_DOWNLOAD_NUM:
+                        raise Exception("%d geocaches found, please select a smaller part of the map!" % count)
                 
                 
-            # Extract waypoint information from the page
-            w = [(
-                # Get the GUID from the link
-                x.getparent().getchildren()[0].get('href').split('guid=')[1], 
-                # See whether this cache was found or not
-                'TertiaryRow' in x.getparent().getparent().get('class'), 
-                # Get the GCID from the text
-                x.text_content().split('|')[1].strip()
-                ) for x in doc.cssselect(".SearchResultsTable .Merge .small")]
-            wpts += w
+                    # Extract waypoint information from the page
+                    w = [(
+                        # Get the GUID from the link
+                        # Actually, this is not the GUID anymore...
+                        x.getparent().getchildren()[0].get('href').split('/')[-1],
+                        # See whether this cache was found or not
+                        'TertiaryRow' in x.getparent().getparent().get('class'),
+                        # Get the GCID from the text
+                        x.text_content().split('|')[1].strip()
+                        ) for x in doc.cssselect(".SearchResultsTable .Merge .small")]
+                    wpts += w
+
                 
             cont = False  
             # There are more pages...
@@ -233,7 +251,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
         points_that_need_downloading = [] # Geocaches that need to be downloaded
         points_finished = []              # Geocaches that only need to be updated in the database
         # and Geocaches which don't need any update (they will be removed)
-        for guid, found, id in wpts:
+        for fullname, found, id in wpts:
             # Check if geocache exists in DB
             coordinate = get_geocache_callback(id)
             
@@ -244,7 +262,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
                     logger.info("Skipping %s. It was not in the DB." % id)
                     continue
                 # Else always download
-                points_that_need_downloading.append((guid, found, id, GeocacheCoordinate(-1, -1, id)))
+                points_that_need_downloading.append((fullname, found, id, GeocacheCoordinate(-1, -1, id)))
                 logger.info("Downloading %s. It was not in the DB." % id)
                 continue
             
@@ -264,21 +282,21 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 logger.info("Updating %s. It was in the DB, but its found status was not correct." % id)
                 continue
             logger.info("Downloading %s. It was in the DB, but it was not to be skipped." % id)
-            points_that_need_downloading.append((guid, found, id, coordinate))
+            points_that_need_downloading.append((fullname, found, id, coordinate))
                     
         
         # Download the geocaches using the print preview 
 
         i = 0
-        for guid, found, id, coordinate in points_that_need_downloading:
+        for fullname, found, id, coordinate in points_that_need_downloading:
             i += 1
             coordinate.found = found
             logger.debug("Coordinate %s, found=%r" % (coordinate.name, found))
             self.emit("progress", "Geocache %d of %d" % (i, len(points_that_need_downloading)), i, len(points_that_need_downloading))
             logger.info("Downloading %s..." % id)
-            url = self.PRINT_PREVIEW_URL % guid
+            url = self.PRINT_PREVIEW_URL_FULLNAME % fullname
             response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)                
-            result = self._parse_cache_page_print(response, coordinate, num_logs = 20)
+            result = self.__parse_cache_page(response, coordinate, 10, progress_min = i, progress_max = i+1, progress_all = len(points_that_need_downloading), download_images = False)
             if result != None and result.lat != -1:
                 points_finished.append(result)
                 
@@ -295,14 +313,15 @@ class GeocachingComCacheDownloader(CacheDownloader):
         url = self.DETAILS_URL % coordinate.name
         response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
             
-        return self._parse_cache_page(response, coordinate, num_logs, progress_min = progress_min, progress_max = progress_max, progress_all = progress_all)
+        return self.__parse_cache_page(response, coordinate, num_logs, progress_min = progress_min, progress_max = progress_max, progress_all = progress_all)
 
     
     @staticmethod
     def check_login_callback(downloader):
         login_request = downloader.get_reader(GeocachingComCacheDownloader.NEAREST_URL, login = False)
-        page = login_request.read()
-        login_request.close()
+        page = read_from_network(login_request)
+        if page == None:
+            return False
 
         t = unicode(page, 'utf-8')
         doc = fromstring(t)
@@ -325,8 +344,9 @@ class GeocachingComCacheDownloader(CacheDownloader):
             '__EVENTARGUMENT': ''
         }
         request = downloader.get_reader(GeocachingComCacheDownloader.LOGIN_URL, values, login = False)
-        page = request.read()
-        request.close()
+        page = read_from_network(request)
+        if page == None:
+            return False
 
         t = unicode(page, 'utf-8')
         doc = fromstring(t)
@@ -338,33 +358,32 @@ class GeocachingComCacheDownloader(CacheDownloader):
             return True
         raise Exception("Name/Password MAY be correct, but I encountered unexpected data while logging in.")
         
-    def _get_user_token(self):
-        page = self.downloader.get_reader(self.USER_TOKEN_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
-        for line in page:
-            if line.startswith('vxar uvtoken'):
-                user_token[0] = re.sub('[^A-Z0-9]+', '', line.split('=')[-1])
-                page.close()
-                return
-        logger.error("Using fallback for user token search!")
-        try:
-            page = self.downloader.get_reader(self.USER_TOKEN_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
-            t = page.read()
-            page.close()
-            user_token[0] = re.search('[A-Z0-9]{128}', t).group(0)
-        except Exception:
-            raise Exception("Website contents unexpected. Please check connection.")
-    
-    def _parse_cache_page(self, cache_page, coordinate, num_logs, download_images = True, progress_min = 0.0, progress_max = 1.0, progress_all = 1.0):
+    def __parse_cache_page(self, cache_page, coordinate, num_logs, download_images = True, progress_min = 0.0, progress_max = 1.0, progress_all = 1.0):
         logger.debug("Start parsing, pmin = %f, pmax = %f." % (progress_min, progress_max))
-        pg = cache_page.read()
-        cache_page.close()
+        if cache_page == None:
+            logger.warning("Can't parse this cache, it is None")
+            return
+        pg = read_from_network(cache_page)
+        if pg == None:
+            return False
+
         t = unicode(pg, 'utf-8')
         doc = fromstring(t)
                 
         # Basename - Image name without path and extension
         def basename(url):
             return url.split('/')[-1].split('.')[0]
-            
+
+        #check for premium-only cache
+        try:
+            doc.get_element_by_id('ctl00_ContentBody_basicMemberMsg') #Premium user will not get this
+            logger.info("PREMIUM ONLY cache (and not premium account), skip it totally")
+            return
+        except Exception, e:
+            None
+            #print "normal cache, can continue"
+
+
         # Title
         try:
             coordinate.title = doc.cssselect('meta[name="og:title"]')[0].get('content')
@@ -384,7 +403,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
         try:
             coordinate.websitelink = doc.get_element_by_id('ctl00_ContentBody_uxCacheUrl').get('href')
         except KeyError, e:
-            logger.info("No website link found, skipping.")
+            #logger.info("No website link found, skipping.")
             coordinate.websitelink = ''
         
         # Short Description - Long Desc. is added after the image handling (see below)
@@ -417,7 +436,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
             
         # Terrain/Difficulty
         try:
-            coordinate.difficulty, coordinate.terrain = [self._handle_stars(basename(x.get('src'))) for x in doc.cssselect('.CacheStarImgs span img')]
+            coordinate.difficulty, coordinate.terrain = [self._handle_stars(basename(x.get('src'))) for x in doc.cssselect('.CacheStarLabels img')]
         except Exception, e:
             logger.error("Could not find/parse star ratings")
             
@@ -456,14 +475,25 @@ class GeocachingComCacheDownloader(CacheDownloader):
         
         # User token and Logs
         userToken = ''
+        new_set_of_logs = []
         for x in doc.cssselect('script'):
             if not x.text:
                 continue
             s = x.text.strip()
-            if s.startswith('//<![CDATA[\r\nvar uvtoken'):
-                userToken = re.sub("(?s).*userToken = '", '', s)
-                userToken = re.sub("(?s)'.*", '', userToken)
-                logger.debug("userToken: %s" % userToken)
+            if not s.startswith('//<![CDATA['):
+                continue
+            for line in s.split('\n'):
+                if line.startswith('userToken'):
+                    userToken = re.sub("(?s).*userToken = '", '', line)
+                    userToken = re.sub("(?s)'.*", '', userToken)
+                    logger.debug("userToken: %s" % userToken)
+                    break
+                if line.startswith('initialLogs'):
+                    new_set_of_logs, dummy = self._parse_logs_json(line[13:])
+
+        if userToken == '':
+            logger.error('User token is empty before retrieving logs!')
+                        
         
         self.emit('progress', 'Fetching logs', progress_min + 0.2 * (progress_max - progress_min), progress_all)
         
@@ -472,37 +502,39 @@ class GeocachingComCacheDownloader(CacheDownloader):
             self.LOGBOOK_URL % (userToken, 1),
             login_callback = self.login_callback, 
             check_login_callback = self.check_login_callback)
-        new_set_of_logs, total_page = self._parse_logs_json(logs.read()) #True=we want also number of page
-        logs.close()
-        page_of_logs=num_logs/10 #num_logs from parameter (which comes from settings 'download_num_logs')
+        new_set_of_logs.extend(self._parse_logs_json(read_from_network(logs)))
 
         #First page is already handled, so counter starts from 2
         counter = 2
-        upper_limit = min(total_page, page_of_logs)
-        while (counter <= upper_limit):
+        while len(new_set_of_logs) < num_logs:
             # We want progress to be between 0.3 and 0.5 times of our own range.
             # Our own range is progress_min -> progress_max
             # Log range is 0.3 to 0.5
-            progress_internal = 0.3 + (float(counter-1)/float(upper_limit)) * 0.2
+            progress_internal = 0.3 + (float(len(new_set_of_logs))/float(num_logs)) * 0.2
             logger.debug("- Progress internal is %f" % progress_internal)
             logger.debug("- Progress is %f of %f" % (progress_min + progress_internal * (progress_max - progress_min), progress_all))
-            self.emit('progress', "Logs (%d/%d)" % (counter, upper_limit), progress_min + progress_internal * (progress_max - progress_min), progress_all)
+            self.emit('progress', "Logs (%d)" % counter, progress_min + progress_internal * (progress_max - progress_min), progress_all)
             logs = self.downloader.get_reader(
                 self.LOGBOOK_URL % (userToken, counter),
                 login_callback = self.login_callback, 
                 check_login_callback = self.check_login_callback)
-            new_set_of_logs.extend(self._parse_logs_json(logs.read())[0])
-            logs.close()
+            new_set_of_logs.extend(self._parse_logs_json(read_from_network(logs)))
 
             counter += 1
+            
+            # Bail out if we don't download any logs.
+            if counter > num_logs:
+                logger.error("Downloading logs did not really work.")
+                logger.warning("cache=" +str(coordinate.serialize()))
+                break
 
         coordinate.set_logs(new_set_of_logs)
 
         # Attributes
-        '''if not coordinate.attributes:
+        if not coordinate.attributes:
             logger.warn("No attributes set for this geocache yet.")
             try:
-                attr_xml = doc.cssselect('.CacheDetailNavigationWidget.BottomSpacing .WidgetBody img')
+                attr_xml = doc.cssselect('.CacheDetailNavigationWidget.TopSpacing.BottomSpacing .WidgetBody img')
                 attributes = self._parse_attributes_from_doc(attr_xml)
                 coordinate.clear_attributes()
                 for x in attributes: 
@@ -513,7 +545,6 @@ class GeocachingComCacheDownloader(CacheDownloader):
             except Exception, e:
                 logger.error("Could not find/parse attributes")
                 raise e
-        '''     
         
         # Image Handling
 
@@ -535,7 +566,10 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 return images[url]['filename']
             
             # If this URL is encountered for the first time, calculate the filename
-            ext = url.rsplit('.', 1)[1]
+            if '.' in url: #this is the normal case (filename has dot)
+                ext = url.rsplit('.', 1)[1]    #this will be out of range if no dot in filename
+            else:   #there are filenames without dot and extension
+                ext = "img"
             if not re.match('^[a-zA-Z0-9]+$', ext):
                 ext = 'img'
             filename = "%s-image%d.%s" % (coordinate.name, len(images), ext)
@@ -557,6 +591,9 @@ class GeocachingComCacheDownloader(CacheDownloader):
         # Next, search image elements and replace them
         for element, attribute, link, pos in desc.iterlinks():
             if element.tag == 'img':
+                if attribute is not 'src': #only handle src-fields, it could be e.g. ImageMap
+                    break
+
                 replace_id = found_image(link, element.get('alt') or element.get('title') or '')
                 replacement = '[[img:%s]]' % replace_id
                 for parent in desc.getiterator():
@@ -583,13 +620,21 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 logger.info("Downloading %s to %s" % (url, filename))
                 
                 # Download file
+                #Some images (e.g. smileys) have path without domain name
+                if url.startswith("/"):
+                    url = "http://www.geocaching.com"+url  #Needs rethinking when agtl supports different domains/services.
+
+                #some images (e.g. attribute-icons) have path without domain name
+                if url.startswith("../"):
+                    url = "http://www.geocaching.com"+url[2:]  #Needs rethinking when agtl supports different domains/services.
+
                 try:
                     f = open(filename, 'wb')
-                    f.write(self.downloader.get_reader(url, login = False).read())
+                    f.write(read_from_network(self.downloader.get_reader(url, login = False)))
                     f.close()
                 except Exception, e:
-                    logger.exception(e)
-                    logger.error("Failed to download image from URL %s" % url)
+                    #logger.exception(e)
+                    logger.error("Failed to download image from URL '%s': '%s'" % (url,e))
                 counter += 1
                 
         # And save Images to coordinate
@@ -620,10 +665,12 @@ class GeocachingComCacheDownloader(CacheDownloader):
         
     # This parses the print preview of a geocache
     # It currently omits images, waypoints and logs.
-    def _parse_cache_page_print(self, cache_page, coordinate, num_logs):
+    def __parse_cache_page_print(self, cache_page, coordinate, num_logs):
         logger.debug("Start parsing.")
-        pg = cache_page.read()
-        cache_page.close()
+        pg = read_from_network(cache_page)
+        if pg == None:
+           return None
+
         t = unicode(pg, 'utf-8')
         doc = fromstring(t)
         
@@ -643,12 +690,12 @@ class GeocachingComCacheDownloader(CacheDownloader):
             raise e  
             
         # Type 
-        try:
-            t = int(basename(doc.cssselect('#Content h2 img')[0].get('src')).split('.')[0])
-            coordinate.type = self.CTIDS[t] if t in self.CTIDS else GeocacheCoordinate.TYPE_UNKNOWN
-        except Exception, e:
-            logger.error("Could not find type - probably premium cache!")
-            return None
+        #try:
+        t = int(basename(doc.cssselect('#Content h2 img')[0].get('src')).split('.')[0])
+        coordinate.type = self.CTIDS[t] if t in self.CTIDS else GeocacheCoordinate.TYPE_UNKNOWN
+        #except Exception, e:
+        #    logger.error("Could not find type - probably premium cache!")
+        #    return None
         
         # Short Description - Long Desc. is added after the image handling (see below)
         try:
@@ -772,7 +819,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
 
                 try:
                     f = open(filename, 'wb')
-                    f.write(self.downloader.get_reader(url, login = False).read())
+                    f.write(read_from_network(self.downloader.get_reader(url, login = False)))
                     f.close()
                 except Exception, e:
                     logger.exception(e)
@@ -782,114 +829,169 @@ class GeocachingComCacheDownloader(CacheDownloader):
             attributes.append(attrib)
         return attributes
         
+    def _upload_fieldnotes_and_logs(self, geocaches):
+        '''
+        Upload all fieldnotes and logs of the geocaches given in the first argument and return a list of geocaches, for which uploading worked.
+        
+        '''
+        upload_fieldnotes = []
+        upload_logs = []
+        for geocache in geocaches:
+            if geocache.upload_as == GeocacheCoordinate.UPLOAD_AS_LOG:
+                upload_logs.append(geocache)
+            else:
+                upload_fieldnotes.append(geocache)
+                
+                
+        logger.info("Will try to upload %d fieldnotes and %d logs." % (len(upload_fieldnotes), len(upload_logs)))
+        
+        success_fieldnotes = self.__upload_fieldnotes(upload_fieldnotes)
+        success_logs = self.__upload_logs(upload_logs)
+        
+        before = len(geocaches)
+        after = len(success_fieldnotes) + len(success_logs)
+        
+        if after == 0 and before != 0:
+            self.emit("download-error", Exception("Uploading of fieldnotes and logs failed."))
+        elif after < before:
+            self.emit("download-error", Exception("Uploading failed for some fieldnotes or logs."))
+        
+        return success_fieldnotes + success_logs
+                        
+        
     # Upload one or more fieldnotes
-    def _upload_fieldnotes(self, geocaches):
+    def __upload_fieldnotes(self, geocaches):
         notes = []
+        if len(geocaches) == 0:
+            return []
         logger.info("Preparing fieldnotes (new downloader)...")
-        for geocache in geocaches:
-            if geocache.logdate == '':
-                raise Exception("Illegal Date.")
+        try:
+            for geocache in geocaches:
+                name, logdate, logtype, fieldnotes = geocache.name, geocache.logdate, geocache.logas, geocache.fieldnotes
+                if logdate == '':
+                    raise Exception("Illegal Date.")
+                
+                try:
+                    logtype_trans = self.TRANS_FIELDNOTE_TYPE[logtype]
+                except KeyError, e:
+                    raise Exception("Illegal status: %s" % logtype)
 
-            if geocache.logas == GeocacheCoordinate.LOG_AS_FOUND:
-                log = "Found it"
-            elif geocache.logas == GeocacheCoordinate.LOG_AS_NOTFOUND:
-                log = "Didn't find it"
-            elif geocache.logas == GeocacheCoordinate.LOG_AS_NOTE:
-                log = "Write note"
-            else:
-                raise Exception("Illegal status: %s" % geocache.logas)
+                text = fieldnotes.replace('"', "'")
 
-            text = geocache.fieldnotes.replace('"', "'")
-
-            notes.append('%s,%sT10:00Z,%s,"%s"' % (geocache.name, geocache.logdate, log, text))
+                notes.append('%s,%sT10:00Z,%s,"%s"' % (name, logdate, logtype_trans, text))
+                
+            logger.info("Uploading fieldnotes...")
             
-        logger.info("Uploading fieldnotes...")
-        
-        self.emit('progress', "Uploading Fieldnotes (Step 1 of 2)", 0, 2)
-        
-        # First, download webpage to get the correct viewstate value
-        cache_page = self.downloader.get_reader(self.UPLOAD_FIELDNOTES_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
-        pg = cache_page.read()
-        cache_page.close()
-        t = unicode(pg, 'utf-8')
-        doc = fromstring(t)
-        # Sometimes this field is not available
-        if 'ctl00$ContentBody$chkSuppressDate' in doc.forms[0].fields:
-            doc.forms[0].fields['ctl00$ContentBody$chkSuppressDate'] = ''
-        values = doc.forms[0].form_values()
-        values += [('ctl00$ContentBody$btnUpload', 'Upload Field Note')]
-        content = '\r\n'.join(notes).encode("UTF-16")
-        
-        data = self.downloader.encode_multipart_formdata(values, [('ctl00$ContentBody$FieldNoteLoader', 'geocache_visits.txt', content)])
-        self.emit('progress', "Uploading Fieldnotes (Step 2 of 2)", 1, 2)
-        response = self.downloader.get_reader(self.UPLOAD_FIELDNOTES_URL, 
-            data=data, 
-            login_callback = self.login_callback, 
-            check_login_callback = self.check_login_callback)
-
-        res = response.read()
-        response.close()
-        t = unicode(res, 'utf-8')
-        doc = fromstring(t)
-        
-        # There's no real success/no success message on the website. 
-        # We therefore assume success, if this element is in the response
-        if doc.get_element_by_id('ctl00_ContentBody_lnkFieldNotes', None) == None:
-            raise Exception("Something went wrong while uploading the field notes.")
-        else:
-            logger.info("Finished upload!")
-       
-    # Upload one or more logs
-    def _upload_logs(self, geocaches):
-        logger.info("Preparing logs...")
-        for geocache in geocaches:
-            if geocache.logdate == '':
-                raise Exception("Illegal Date.")
-
-            if geocache.logas == GeocacheCoordinate.LOG_AS_FOUND:
-                log = 2
-            elif geocache.logas == GeocacheCoordinate.LOG_AS_NOTFOUND:
-                log = 3
-            elif geocache.logas == GeocacheCoordinate.LOG_AS_NOTE:
-                log = 4
-            else:
-                raise Exception("Illegal status: %s" % geocache.logas)
-
-            text = geocache.fieldnotes
-            year, month, day = geocache.logdate.split('-')
-            
-            url = 'http://www.geocaching.com/seek/log.aspx?wp=%s' % geocache.name
+            self.emit('progress', "Uploading Fieldnotes (Step 1 of 2)", 0, 2)
             
             # First, download webpage to get the correct viewstate value
-            cache_page = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
-            pg = cache_page.read()
-            cache_page.close()
+            cache_page = self.downloader.get_reader(self.UPLOAD_FIELDNOTES_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+            pg = read_from_network(cache_page)
+            if pg == None:
+                logger.error("network error")
+                return
             t = unicode(pg, 'utf-8')
             doc = fromstring(t)
-            doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$ddLogType'] = str(log)
-            doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$DateTimeLogged$Day'] = str(int(day))
-            doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$DateTimeLogged$Month'] = str(int(month))
-            doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$DateTimeLogged$Year'] = str(int(year))
-            doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$uxLogInfo'] = text
-            values = dict(doc.forms[0].form_values())
-            values['ctl00$ContentBody$LogBookPanel1$LogButton'] = doc.get_element_by_id('ctl00_ContentBody_LogBookPanel1_LogButton').get('value')
-            logger.debug("Field values are %r" % values)
-            response = self.downloader.get_reader(url, 
-                values=values, 
+            # Sometimes this field is not available
+            if 'ctl00$ContentBody$chkSuppressDate' in doc.forms[0].fields:
+                doc.forms[0].fields['ctl00$ContentBody$chkSuppressDate'] = ''
+            values = doc.forms[0].form_values()
+            values += [('ctl00$ContentBody$btnUpload', 'Upload Field Note')]
+            content = '\r\n'.join(notes).encode("UTF-16")
+            
+            data = self.downloader.encode_multipart_formdata(values, [('ctl00$ContentBody$FieldNoteLoader', 'geocache_visits.txt', content)])
+            self.emit('progress', "Uploading Fieldnotes (Step 2 of 2)", 1, 2)
+            response = self.downloader.get_reader(self.UPLOAD_FIELDNOTES_URL, 
+                data = data, 
                 login_callback = self.login_callback, 
                 check_login_callback = self.check_login_callback)
 
-            res = response.read()
-            res.close()
+            res = read_from_network(response)
+            if res == None:
+                logger.error("network error")
+                return
+
             t = unicode(res, 'utf-8')
             doc = fromstring(t)
             
             # There's no real success/no success message on the website. 
             # We therefore assume success, if this element is in the response
-            if doc.get_element_by_id('ctl00_ContentBody_LogBookPanel1_ViewLogPanel', None) == None:
-                raise Exception("Something went wrong while uploading the log.")
+            if doc.get_element_by_id('ctl00_ContentBody_lnkFieldNotes', None) == None:
+                raise Exception("Something went wrong while uploading the field notes.")
             else:
                 logger.info("Finished upload!")
+            return geocaches
+        except Exception, e:
+            logger.exception(e)
+        
+            
+       
+    # Upload one or more logs
+    def __upload_logs(self, geocaches):
+        if len(geocaches) == 0:
+            return []
+        success = []
+        logger.info("Preparing logs...")
+        i = 0
+        for geocache in geocaches:
+            try:
+                name, logdate, logtype, fieldnotes = geocache.name, geocache.logdate, geocache.logas, geocache.fieldnotes
+                
+                if logdate == '':
+                    raise Exception("Illegal Date.")
+
+                try:
+                    logtype_trans = self.TRANS_LOG_TYPE[logtype]
+                except KeyError, e:
+                    raise Exception("Illegal status: %s" % logtype)
+                
+                year, month, day = logdate.split('-')
+                
+                url = self.UPLOAD_LOG_URL % name
+                
+                # First, download webpage to get the correct viewstate value
+                self.emit('progress', "Uploading Logs (%d of %d)..." % (i, len(geocaches)), i + 1, len(geocaches))
+                cache_page = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+                pg = read_from_network(cache_page)
+                if pg == None:
+                    return success #it is list of successfully
+                t = unicode(pg, 'utf-8')
+                doc = fromstring(t)
+                doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$ddLogType'] = str(logtype_trans)
+                doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$DateTimeLogged$Day'] = str(int(day))
+                doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$DateTimeLogged$Month'] = str(int(month))
+                doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$DateTimeLogged$Year'] = str(int(year))
+                doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$uxLogInfo'] = fieldnotes
+                values = dict(doc.forms[0].form_values())
+                values['ctl00$ContentBody$LogBookPanel1$LogButton'] = doc.get_element_by_id('ctl00_ContentBody_LogBookPanel1_LogButton').get('value')
+                
+                # Now, manually re-encoding the text
+                # Normally, get_reader (below) expects unicode-encoded values, but lxml (above) does not allow that.
+                values['ctl00$ContentBody$LogBookPanel1$uxLogInfo'] = values['ctl00$ContentBody$LogBookPanel1$uxLogInfo'].encode('utf-8')
+                
+                response = self.downloader.get_reader(url, 
+                    values = values, 
+                    login_callback = self.login_callback, 
+                    check_login_callback = self.check_login_callback)
+
+                res = read_from_network(response)
+                if res == None:
+                    return success #it is list of successfully
+                t = unicode(res, 'utf-8')
+                doc = fromstring(t)
+                
+                # There's no real success/no success message on the website. 
+                # We therefore assume success, if this element is in the response
+                if doc.get_element_by_id('ctl00_ContentBody_LogBookPanel1_ViewLogPanel', None) == None:
+                    raise Exception("Something went wrong while uploading the log.")
+                else:
+                    logger.info("Finished upload!")
+                i += 1
+                success.append(geocache)
+            except Exception, e:
+                logger.exception(e)
+        return success
+            
                  
     # Only return the contents of a node, not the node tag itself, as text
     def _extract_node_contents(self, el):
@@ -927,6 +1029,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
         
     def _parse_logs_json(self, logs):
         logger.debug("Start json logs parsing")
+        #print logs
         try:
             r = json.loads(logs)
         except Exception, e:
@@ -935,18 +1038,18 @@ class GeocachingComCacheDownloader(CacheDownloader):
         if not 'status' in r or r['status'] != 'success':
             logger.error('Could not read logs, status is "%s"' % r['status'])
         data = r['data']
-
         output = []
         for l in data:
-            tpe = l['LogTypeImage'].replace('.gif', '').replace('icon_', '')
+            tpe = l['LogType']
+            icon = l['LogTypeImage']
             date = l['Visited']
             finder = "%s (found %s)" % (l['UserName'], l['GeocacheFindCount'])
             text = HTMLManipulations._decode_htmlentities(HTMLManipulations._strip_html(HTMLManipulations._replace_br(l['LogText'])))
-            output.append(dict(type=tpe, date=date, finder=finder, text=text))
+            output.append(dict(type=tpe, typeicon=icon, date=date, finder=finder, text=text))
         logger.debug("Read %d log entries" % len(output))
         
         total_page = r['pageInfo']['totalPages']
-        return output,total_page
+        return output
 
 BACKENDS = {
     'geocaching-com-new': {'class': GeocachingComCacheDownloader, 'name': 'geocaching.com', 'description': 'Backend for geocaching.com'},
