@@ -72,9 +72,6 @@ if '--simple' in argv:
     import simplegui
     gui = simplegui.SimpleGui
     gps = 'gpsdprovider'
-elif '--desktop' in argv:
-    import biggui
-    gui = biggui.BigGui
 elif '--qml' in argv:
     import qmlgui
     gui = qmlgui.QmlGui
@@ -171,6 +168,8 @@ class Core(gobject.GObject):
         'debug_log_to_http': False,
         'options_backend': 'geocaching-com-new',
         'options_redownload_after': 14,
+        'use_assisted_gps': True,
+        'gps_interval_seconds': 1,
     }
             
     def __init__(self, guitype, gpstype, extensions):
@@ -206,12 +205,38 @@ class Core(gobject.GObject):
         self.create_recursive(self.settings['download_map_path'])
         
         self.downloader = downloader.FileDownloader(self.settings['options_username'], self.settings['options_password'], self.COOKIE_FILE)
-                
+
+        # Download log-type icons
+        #logger.info("Download log-type icons")
+        import os
+        filename = os.path.join(self.settings['download_output_dir'], "1.png" )
+        if os.path.isfile(filename):
+            logger.info("%s exists already, don't reload icons for log-types" % filename)
+        else:
+            #server have mystery ranges of filenames
+            icons = range(1,26) + range(45,49) + range(54,58) + range(67,78)
+            #for some reason some numbers are not used
+            icons.remove(8)
+            icons.remove(20)
+            icons.remove(21)
+
+            for i in icons:
+                filename = os.path.join(self.settings['download_output_dir'], "%s.png" % i)
+                url="http://www.geocaching.com/images/logtypes/%s.png" % i
+                logger.info("Downloading icon for log-type %s to %s" % (url, filename))
+
+                try:
+                    f = open(filename, 'wb')
+                    f.write(downloader.read_from_network(self.downloader.get_reader(url, login = False)))
+                    f.close()
+                except Exception, e:
+                    logger.exception(e)
+                    logger.error("Failed to download image from URL %s" % url)
+
         self.pointprovider = provider.PointProvider(self.CACHES_DB, geocaching.GeocacheCoordinate)
 
         self.gui = guitype(self)
         
-        
         if ('debug_log_to_http' in self.settings and self.settings['debug_log_to_http']) or '--remote' in argv:
             http_handler = logging.handlers.HTTPHandler("danielfett.de", "http://www.danielfett.de/files/collect.php")
             buffering_handler = logging.handlers.MemoryHandler(100, target = http_handler)
@@ -598,7 +623,9 @@ class Core(gobject.GObject):
             self.gps_thread = gpsreader.GpsReader()
             gobject.timeout_add(1000, lambda: self.__read_gps(self.gps_thread.get_data()))
         elif gps == 'locationgpsprovider':
-            self.gps_thread = gpsreader.LocationGpsReader(self.__read_gps)
+            use_agps = self.settings['use_assisted_gps']
+            gps_interval = self.settings['gps_interval_seconds']
+            self.gps_thread = gpsreader.LocationGpsReader(self.__read_gps, use_agps,gps_interval)
             gobject.idle_add(self.gps_thread.start)
         elif gps == 'qmllocationprovider':
             self.gui.get_gps(self.__read_gps)
@@ -862,7 +889,7 @@ class Core(gobject.GObject):
             t.start()
             return False
         else:
-            return self._download_cache_details_list_complete(self.cachedownloader.update_coordinates(caches))
+            return self._download_cache_details_list_complete(self.cachedownloader.update_coordinates(caches,self.settings['download_num_logs']))
         
     def _download_cache_details_list_complete(self, caches):
         """
@@ -992,23 +1019,27 @@ class Core(gobject.GObject):
         elif cache.logas == geocaching.GeocacheCoordinate.LOG_AS_NOTFOUND:
             cache.found = 0
 
-        self.save_cache_attribute(cache, ('logas', 'logdate', 'fieldnotes', 'found'))
+        self.save_cache_attribute(cache, ('logas', 'logdate', 'fieldnotes', 'found', 'upload_as'))
         self.emit('fieldnotes-changed')        
+        self.emit('cache-changed', cache)
 
-    def upload_fieldnotes(self):
+    def upload_fieldnotes(self, sync = False):
         """
-        Upload fieldnotes to the web site. 
+        Upload fieldnotes and logs to the web site. 
         
         """
         caches = self.pointprovider.get_new_fieldnotes()
-        
-        t = Thread(target=self._download_upload_helper, args=['self.cachedownloader.upload_fieldnotes', self._upload_fieldnotes_complete, caches])
-        t.daemon = True
-        t.start()
+        if not sync:
+            t = Thread(target=self._download_upload_helper, args=['self.cachedownloader.upload_fieldnotes_and_logs', self._upload_fieldnotes_complete, caches])
+            t.daemon = True
+            t.start()
+        else:
+            res = self.cachedownloader.upload_fieldnotes_and_logs(caches)
+            self._upload_fieldnotes_complete(res)
         
     def _upload_fieldnotes_complete(self, caches):
         """
-        Called when uploading of fieldnotes is complete.
+        Called when uploading of fieldnotes and logs is complete.
         
         Resets the fieldnotes which were uploaded to "NO LOG".
         """
@@ -1092,14 +1123,14 @@ def start_profile(what):
 
     
     print "BY TOTALTIME:\n------------------------------------------------------------"
-    def c(x, y):
+    def c2(x, y):
         if x.totaltime < y.totaltime:
             return 1
         elif x.totaltime == y.totaltime:
             return 0
         else:
             return -1
-    stats.sort(cmp=c)
+    stats.sort(cmp=c2)
     for line in stats[:30]:
         print "%d %4f %s" % (line.callcount, line.totaltime, line.code)
         if line.calls == None:
